package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ExportDiagnoseDTO;
import com.diagbot.entity.KlDiagnoseDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.KlDiagnoseDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 诊断依据明细表 服务实现类
 * </p>
 *
 * @author zhoutg
 * @since 2021-06-10
 */
@Service
public class KlDiagnoseDetailFacade extends KlDiagnoseDetailServiceImpl {

    @Autowired
    KlDiagnoseDetailFacade klDiagnoseDetailFacade;

    /**
     * 导出诊断依据【初版】
     */
    public void exportDiagnose(HttpServletResponse response) {
        // 获取全部数据、分组
        List<KlDiagnoseDetail> klDiagnoseDetailList = klDiagnoseDetailFacade.list(new QueryWrapper<KlDiagnoseDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap = EntityUtil.makeEntityListMap(klDiagnoseDetailList, "disName");
        // TODO 获取公式，分隔编码，替换编码，组装导出数据  Map<String, List<ExportDiagnoseDTO>>
        Map<String, List<ExportDiagnoseDTO>> diagnoseDetails = processDiagnoseDetail(diagnoseDetailMap);
        System.out.println();


        // 导出数据
        List<Map<String, Object>> exportDto = new LinkedList<>();

        List<ExportDiagnoseNameDTO> exportDiagnoseNameDTOList = getCatalogue(diagnoseDetails); // 生成目录名称
        Map<String, String> styleMap = new LinkedHashMap<>();
        styleMap.put(ExcelUtils.HYPERLINK, "com.diagbot.entity.ExcelDataHandler");
        exportDto.add(ExcelUtils.createOneSheet("目录", null, ExportDiagnoseNameDTO.class, exportDiagnoseNameDTOList, null));

        int i = 0;
        for (String key : diagnoseDetails.keySet()) {
            if (i > 10) {
                break;
            }
            exportDto.add(ExcelUtils.createOneSheet(key, null, ExportDiagnoseDTO.class, diagnoseDetails.get(key), null));
            i++;
        }
        ExcelUtils.exportExcel(exportDto, "diagnose.xls", response);
    }


    public Map<String, List<ExportDiagnoseDTO>> processDiagnoseDetail(Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap){
        Map<String, List<ExportDiagnoseDTO>> data = new LinkedHashMap<>();
        for(Map.Entry<String, List<KlDiagnoseDetail>> kd : diagnoseDetailMap.entrySet()){
            String sheetName = kd.getKey();
            List<KlDiagnoseDetail> datats = kd.getValue();
            removeKlDiagnoseDetail(datats);
            List<ExportDiagnoseDTO> eds = Lists.newArrayList();
            for (KlDiagnoseDetail kdl:datats) {
                Integer type = kdl.getType();
                String code = kdl.getCode();
                ExportDiagnoseDTO exportDiagnoseDTO = new ExportDiagnoseDTO();
                BeanUtil.copyProperties(kdl, exportDiagnoseDTO);
                ReflectUtil.setProperty(exportDiagnoseDTO, "type", type.toString());
                ReflectUtil.setProperty(exportDiagnoseDTO, "orderNo", code);
                if(code.startsWith("3.") || code.startsWith("4.")){
                    exportDiagnoseDTO.setRule(kdl.getRelation());
                    exportDiagnoseDTO.setRelation("");
                }else {
                    exportDiagnoseDTO.setRule(kdl.getStandard());
                }
                if(type == 91 || type == 92 || type == 93){
                    exportDiagnoseDTO.setRule(kdl.getFormula());
                }

                exportDiagnoseDTO.setEq(kdl.getMidResult());
                exportDiagnoseDTO.setType("1223");
                eds.add(exportDiagnoseDTO);
            }
            data.put(sheetName,eds);
        }
        return data;
    }

    /**
     * 生成目录
     *
     * @param diagnoseDetails
     * @return
     */
    public  List<ExportDiagnoseNameDTO> getCatalogue(Map<String, List<ExportDiagnoseDTO>> diagnoseDetails) {
        List<ExportDiagnoseNameDTO> exportDiagnoseNameDTOList = Lists.newArrayList();
        for (String key : diagnoseDetails.keySet()) {
            ExportDiagnoseNameDTO exportDiagnoseNameDTO = new ExportDiagnoseNameDTO();
            exportDiagnoseNameDTO.setName(key);
            exportDiagnoseNameDTOList.add(exportDiagnoseNameDTO);
        }
        return exportDiagnoseNameDTOList;
    }

    /**
     * 移除公式编码中没有的数据
     * @param klDiagnoseDetails
     */
    public void removeKlDiagnoseDetail(List<KlDiagnoseDetail> klDiagnoseDetails){
        Map<String, List<String>> allFormula = getAllFormula(klDiagnoseDetails);
        List<String> regexData = getCodes(allFormula);
        Iterator<KlDiagnoseDetail> iterator = klDiagnoseDetails.iterator();
        while (iterator.hasNext()){
            KlDiagnoseDetail next = iterator.next();
            String code = next.getCode();
            if(!regexData.contains(code) && StringUtil.isNotBlank(code)){
                iterator.remove();
            }
        }
    }


    /**
     * 获取每个诊断依据的公式
     * @param klDiagnoseDetails
     * @return
     */
    private Map<String,List<String>> getAllFormula(List<KlDiagnoseDetail> klDiagnoseDetails){
        Map<String,List<String>> typeFormulaMap = new HashMap<>();
        for (KlDiagnoseDetail klDiagnoseDetail : klDiagnoseDetails) {
            Integer type = klDiagnoseDetail.getType();
            String formula = klDiagnoseDetail.getFormula();
            if((type == 91 || type == 92 || type == 93)&& StringUtil.isNotBlank(formula)){
                if(typeFormulaMap.containsKey(type.toString())){
                    List<String> formulas = typeFormulaMap.get(type.toString());
                    formulas.add(formula);
                    typeFormulaMap.put(type.toString(),formulas);
                }else {
                    List<String> formulas = Lists.newArrayList();
                    formulas.add(formula);
                    typeFormulaMap.put(type.toString(),formulas);
                }
            }
        }
        return typeFormulaMap;
    }

    /**
     * 获取公式中的所有编码
     * @param getAllFormula
     * @return
     */
    private List<String> getCodes(Map<String,List<String>> getAllFormula) {
        StringBuffer sb = new StringBuffer();
        String pateern = "([1-9]\\d*\\.?\\d*)|(0\\.\\d*[1-9])";
        if(MapUtils.isNotEmpty(getAllFormula)){
            getAllFormula.forEach((x,y) ->{
                String collect = y.stream().collect(Collectors.joining(","));
                sb.append(collect).append("+");
            });
        }
        //获取公式中的编码
        return RegexUtil.getRegexDatas(sb.toString(), pateern);
    }

}
